package cz.jzitnik.game.handlers;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Set;
import java.util.function.Function;

import org.reflections.Reflections;
import org.reflections.scanners.Scanners;
import org.reflections.util.ConfigurationBuilder;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.annotations.MineEventHandler;

public class BreakBlockActionProvider {
    private HashMap<String, Function<Game, Void>> actions = new HashMap<>();

    public BreakBlockActionProvider() {
        register();
    }

    private void register() {
        Reflections reflections = new Reflections(
                new ConfigurationBuilder()
                        .forPackage("cz.jzitnik.game.handlers.events.handlers.mine")
                        .addScanners(Scanners.MethodsAnnotated)
        );
        Set<Method> mineHandlers = reflections.getMethodsAnnotatedWith(MineEventHandler.class);

        for (Method method : mineHandlers) {
            if (method.getParameterCount() == 1 &&
                    method.getParameterTypes()[0] == Game.class)
                try {
                    Object instance = method.getDeclaringClass().getDeclaredConstructor().newInstance();
                    Function<ScreenRenderer, Game, Integer, Integer> handler = (screenRenderer, game, x, y) -> {
                        try {
                            method.invoke(instance, screenRenderer, game, x, y);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    };
                    this.mineHandlers.add(handler);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
